/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util;

import java.io.*;
import java.nio.channels.FileChannel;
import java.util.List;
import java.util.Set;
import org.openide.filesystems.FileObject;

/**
 *
 * @author Merijn Zengers, Gerben Feenstra
 */
public class FileCopier {

    /**
     * Copy whole directory + content to the destination directory
     *
     * @param source The source directory
     * @param destination The destination directory
     * @param extension The file-extension to search for in the sourcedir, for
     * example ".shp". Null will copy all files from source dir to destination
     * @param bundleFiles In conjunction with 'extension': Copy files with same
     * names but different extension also (like: If file is named
     * thisisaname.shp, then copy thisisaname.shx, thisisaname.dbf etc. as well)
     * @throws IOException While creating a file
     * @throws NotADirectoryException The specified destination is not a
     * directory
     * @throws NotAFileException The passed file (in the call to copyFile) is
     * not a file
     * @throws DirectoryCreationException When the destination directory cannot
     * be created
     */
    public static void copyDirectory(File source, File destination, String extension, boolean bundleFiles) throws NotAFileException, NotADirectoryException, DirectoryCreationException, IOException {
        checkAndCreateDirectory(source);
        checkAndCreateDirectory(destination);

        // Not just copy the files from the source-location, but also the selected parent directory.
        File newDestination = new File(destination.getAbsolutePath() + source.getPath().substring(source.getPath().lastIndexOf(File.separator)));
        checkAndCreateDirectory(newDestination);

        // List files in source directory and initiate file copy
        for (File f : source.listFiles()) {
            if (f.isFile() && (extension == null || f.getName().toLowerCase().endsWith(extension))) {
                copyFile(f, newDestination, bundleFiles);
            }
        }
    }

    /**
     * Copies file 'source' to destination directory 'destination'
     *
     * @param source
     * @param destination
     * @param bundleFiles Copy files with the same name but different extension
     * also (like: If file is named thisisaname.shp, then copy thisisaname.shx,
     * thisisaname.dbf etc. as well)
     * @throws NotADirectoryException
     * @throws NotAFileException
     * @throws DirectoryCreationException
     * @throws FileNotFoundException
     * @throws IOException
     */
    public static void copyFile(File source, File destination, boolean bundleFiles) throws NotAFileException, NotADirectoryException, DirectoryCreationException, IOException {
        checkIsFile(source);
        checkAndCreateDirectory(destination);

        if (bundleFiles) {
            // Find other files with the same name in the same path
            for (File f : source.getParentFile().listFiles()) {
                if (f.isFile()) {
                    String fileName = source.getName().substring(0, source.getName().lastIndexOf("."));
                    String bundleFileName = f.getName().substring(0, f.getName().lastIndexOf("."));
                    if (!fileName.equals("") && !bundleFileName.equals("")) {
                        if (fileName.equals(bundleFileName)) {
                            copyFile(f, destination);
                        }
                    }
                }
            }
        } else {
            copyFile(source, destination);
        }
    }

    /**
     * Copy sourcefile to destination directory using FileChannel
     *
     * @param source Source file to copy to destinaion
     * @param destination Destination directory
     */
    public static void copyFile(File source, File destination) throws NotAFileException, NotADirectoryException, DirectoryCreationException, FileNotFoundException, IOException {
        checkIsFile(source);
        checkAndCreateDirectory(destination);

        // init both FileChannels
        FileChannel fileChannelSrc = null;
        FileChannel fileChannelDest = null;

        // Create new destination file in the destination directory
        StringBuilder sb = new StringBuilder(destination.getPath());
        sb.append(File.separator);
        sb.append(source.getName());
        File newFile = new File(sb.toString());

        // Get channels from both source and new destination file
        fileChannelSrc = new FileInputStream(source).getChannel();
        fileChannelDest = new FileOutputStream(newFile).getChannel();

        // Do the actual copy from the sourcefile to the new destination file
        fileChannelDest.transferFrom(fileChannelSrc, 0, fileChannelSrc.size());
        fileChannelDest.close();
        fileChannelSrc.close();
    }

    /**
     * Check if the specified File is not a directory. If it does not exist,
     * create a new file.
     *
     * @param file The File to check
     */
    private static void checkIsFile(File file) throws NotAFileException, IOException {
        if (file.isDirectory()) {
            StringBuilder sb = new StringBuilder("The suplied path [");
            sb.append(file.getPath());
            sb.append("] is not a File, but a directory");
            throw new NotAFileException(sb.toString());
        }
        // Below is BS, since method is only used to check Source files, which must exist in the first place.
//        // Not a Directory, must be a file, so create it if it doesn't exist
//        if (!file.exists()) {
//            file.createNewFile();
//        }
    }

    /**
     * Check if the specified File is not an actual file, but rather a
     * directory. If this File-instance is indeed representing a directory this
     * method will create the directory if it does not actually exist in the
     * filesystem yet.
     *
     * @param directory The directory file to check
     */
    private static void checkAndCreateDirectory(File directory) throws NotADirectoryException, DirectoryCreationException {
        if (directory.isFile()) {
            StringBuilder sb = new StringBuilder("The suplied path [");
            sb.append(directory.getPath());
            sb.append("] is not a Directory");
            throw new NotADirectoryException(sb.toString());
        }
        // Not a File, must be a directory, so create it
        if (!directory.exists()) {
            if (!directory.mkdirs()) {
                StringBuilder sb = new StringBuilder("Failed while creating directory: ");
                sb.append(directory.getPath());
                throw new DirectoryCreationException(sb.toString());
            }
        }

    }

    /**
     * Copy queue to specified dir.
     *
     * @param copyQueue List of files to copy to the specified destination dir.
     * @Param destination FileObject to copy the files to.
     * @throws IOException
     */
    public static void copy(Set<FileObject> copyQueue, FileObject destination) throws IOException {
        for (FileObject file : copyQueue) {
            try {
                org.openide.filesystems.FileUtil.copyFile(file, destination, file.getName());
            } catch (SyncFailedException ex) {
                //Do nothing file already exists
            }
        }
        copyQueue.clear();
    }
}
