/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.filechooser.FileFilter;
import java.util.List;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;

/**
 *
 * @author Gerben Feenstra
 */
public class FilePickerUtil {

    /**
     * Convenience class. Instances of this class can be used ONLY ONCE as is the smart thing to do with filepickers anyways.
     * Adds convenience method to add filefilter using description and list of extensions (List or vararg). 
     * This class will create a FileFilter-object from the specified description and extensions
     * Before actually opening a (multi) dialog it will create and add an all-files filter.
     * 
     * @author Gerben Feenstra
     */
    // TODO: Center on screen: http://stackoverflow.com/questions/5596089/filechooserbuilder-not-displayed-in-center-of-screen
    public static class AgroSenseFilePicker extends FileChooserBuilder {
        
        private List<String> allExtensions = new ArrayList<String>();
        
        public AgroSenseFilePicker(String dirKey) {
            super(dirKey);
        }
        
        /**
         * For example: addFileFilter("Cropcircle", "csv", "txt")
         * @param description
         * @param extensions 
         */
        public void addFileFilter(String description, String... extensions){
            addFileFilter(description, Arrays.asList(extensions));     
        }
        
        public void addFileFilter(String description, List<String> extensions){  
            super.setFileFilter(createSwingFileFilter(extensions, description));
            allExtensions.addAll(extensions);
        }
        
        @Override
        public File showOpenDialog(){
            super.setFileFilter(createSwingFileFilter(allExtensions, "All supported files"));
            return super.showOpenDialog();
        }

        @Override
        public File[] showMultiOpenDialog(){
            super.setFileFilter(createSwingFileFilter(allExtensions, "All supported files"));
            return super.showMultiOpenDialog();
        }    
        
        public FileChooserBuilder setDefaultWorkingDirectory(FileObject fileobject){
            return super.setDefaultWorkingDirectory(org.openide.filesystems.FileUtil.toFile(fileobject));
        }
    } 
        
    /**
     * Creates a filefilter for the specified List of Extensions
     * @param supportedExtensions The List of extensions in the form of .ext (for example: .dbf and .csv) this filefilter should support.
     * @param formatDescription Description of the filter as shown in the selectionbox
     * @return 
     */
    public static FileFilter createSwingFileFilter(final List<String> supportedExtensions, final String formatDescription) {
        return new FileFilter() {

            @Override
            public boolean accept(File f) {
                if (f.isDirectory() && !f.getName().startsWith(".")) {
                    return true;
                }
                if (f.isFile()) {
                    String name = f.getName().toLowerCase();
                    for (String extention : supportedExtensions) {
                        if (name.endsWith(extention)) {
                            return true;
                        }
                    }
                }
                return false;
            }

            @Override
            public String getDescription() {
                return formatDescription;
            }
        };
    }

    /**
     * Create the display String for a filepicker. String in this form: "preceedingText (supportedExtensions String)" 
     * 
     * @param supportedExtensions List of supported extension to tranform to a String
     * @return String Example: "preceedingText (*.dbf, *.csv)"
     */
    public static String getFileFilterDescription(String preceedingText, List<String> supportedExtensions) {
        StringBuilder sb = new StringBuilder();
        sb.append(preceedingText);
        sb.append(" ");
        sb.append(getFileFilterTypeString(supportedExtensions));
        return sb.toString();
    }

    public static String getFileFilterTypeString(String[] supportedExtensions) {
        return getFileFilterTypeString(Arrays.asList(supportedExtensions));
    }
    
    /**
     * Concatenates the List-items into one String surrounded with ( and )
     * Primary use = to build a file filter
     * 
     * @param supportedExtensions List of extensions in the form of .ext (for example: .dbf and .csv) to concatenate
     * @return The String of concatenated String surrounded with ( and ). Example: (*.dbf, *.csv)
     */
    public static String getFileFilterTypeString(List<String> supportedExtensions) {
        StringBuilder sb = new StringBuilder();
        sb.append("(");
        for (String extension : supportedExtensions) {
            sb.append("*");
            sb.append(extension);
            sb.append(", ");
        }

        // remove last , plus its space
        if (sb.length() > 1) {
            sb.delete(sb.length() - 2, sb.length());
        }

        sb.append(")");
        return sb.toString();
    }
}
